import numpy as np

class ex1:
    def __init__(self):
        self.intentos = 0
        pass

    def _RK4(self, f, ti, tf, N, y0):
        t = np.linspace(ti,tf,N)
        h = t[1] - t[0]
        y_RK4 = np.zeros(N)
        y_RK4[0] = y0
        for i in range(N-1):
            g1 = f(t[i]    , y_RK4[i]       )
            g2 = f(t[i]+h/2, y_RK4[i]+h/2*g1)
            g3 = f(t[i]+h/2, y_RK4[i]+h/2*g2)
            g4 = f(t[i]+h  , y_RK4[i]+h*g3  )
            y_RK4[i+1] = y_RK4[i] + h/6*(g1+2*g2+2*g3+g4)
        return (t, y_RK4)

    def hint(self):
        print("Recuerde que para programar RK4 debe calcular primero los g1, g2, g3, g4 y luego ")
        print("actualizar la solución. Recuerde que la función de lado derecho recibe los")
        print("argumentos en el siguiente orden: t (tiempo), E.")

    def check(self, y, f, ti, tf, N, y0):
        comparison = y==np.zeros(N)
        if not comparison.all():
            self.intentos += 1
        correct = True
        t_ref, y_ref = self._RK4(f, ti, tf, N, y0)
        if len(y_ref) != len(y):
            print("Incorrecto, el largo de los arreglos no coincide.")
            print()
            correct = False
        else:
            for i in range(len(y_ref)):
                if abs(y[i]-y_ref[i])>0.01*abs(y_ref[i]):
                    print("Incorrecto, los valores no coinciden.")
                    print()
                    correct = False
                    break
        if correct:
            print("Correcto!")
            print()

        return (t_ref, y_ref)

    def solution(self):
        if self.intentos==0:
            print("Debe intentarlo al menos una vez")
        else:
            print("A continuación, se presenta un código que genera la solución:")
            print()
            print("def RK4(f, ti, tf, N, E0):")
            print("    t = np.linspace(ti,tf,N)")
            print("    h = t[1] - t[0]")
            print("    E_RK4 = np.zeros(N)")
            print("    E_RK4[0] = E0")
            print("    for i in range(N-1):")
            print("        g1 = f(t[i]    , E_RK4[i]       )")
            print("        g2 = f(t[i]+h/2, E_RK4[i]+h/2*g1)")
            print("        g3 = f(t[i]+h/2, E_RK4[i]+h/2*g2)")
            print("        g4 = f(t[i]+h  , E_RK4[i]+h*g3  )")
            print("        E_RK4[i+1] = E_RK4[i] + h/6*(g1+2*g2+2*g3+g4)")
            print("    return (t, E_RK4)")


from google.colab import _message
import numpy as np

class ex1:
    def __init__(self):
        pass

    def hint(self):
        print("Recuerden que debe llamar a solve_ivp con los argumentos f, (a, b), [y0],")
        print("además de restringir el paso máximo a h_prop.")

    def check(self):
        print("Revisando su solución...")
        print()
        nbstr = _message.blocking_request('get_ipynb', request='', timeout_sec=25)
        for i in range(len(nbstr['ipynb']["cells"])):
            if nbstr['ipynb']["cells"][i]["source"][0]=="from scipy.integrate import solve_ivp\n":
                ans = nbstr['ipynb']["cells"][i]["source"][-1]
                if ans.replace(" ","").upper()=="SOL=SOLVE_IVP(F,(A,B),[Y0],MAX_STEP=H_PROP)":
                    print("Correcto!")
                else:
                    print("Incorrecto")

    def solution(self):
        print("sol = solve_ivp(f, (a, b), [y0], max_step=h_prop)")


from google.colab import _message
import numpy as np

class ex1:
    def __init__(self):
        pass

    def hint(self):
        print("Recuerde que debe crear X0 y los argumentos en el mismo orden en que los recibe la función F2.")
        print("Además, debe llamar a solve_ivp con los argumentos F2, (0,T), X0, args=(args,) y específicar el")
        print("método con el argumento method.")

    def check(self):
        pos_answ1 = [
            "X0,ARGS=[A0,M0,1-A0-M0],(K1,K2,ALPHA)",
            "X0,ARGS=[A0,M0,S0],(K1,K2,ALPHA)",
            "X0,ARGS=[A0,M0,1-A0-M0],[K1,K2,ALPHA]",
            "X0,ARGS=[A0,M0,S0],[K1,K2,ALPHA]"
        ]
        pos_answ2 = [
            "SOL=SOLVE_IVP(F2,(0,T),X0,METHOD='RK23',ARGS=(ARGS,))",
            "SOL=SOLVE_IVP(F2,(0,T),X0,METHOD='DOP853',ARGS=(ARGS,))",
            "SOL=SOLVE_IVP(F2,(0,T),X0,METHOD='RADAU',ARGS=(ARGS,))",
            "SOL=SOLVE_IVP(F2,(0,T),X0,METHOD='DBF',ARGS=(ARGS,))",
            "SOL=SOLVE_IVP(F2,(0,T),X0,METHOD='LSODA',ARGS=(ARGS,))"
        ]

        nbstr = _message.blocking_request('get_ipynb', request='', timeout_sec=10)
        for i in range(len(nbstr['ipynb']["cells"])):
            if nbstr['ipynb']["cells"][i]["source"][0]=="from scipy.integrate import solve_ivp\n":
                ans1 = nbstr['ipynb']["cells"][i]["source"][-2]
                ans2 = nbstr['ipynb']["cells"][i]["source"][-1]
                if ans1.replace(" ","").upper() not in pos_answ1:
                    print("Incorrecto, debe ingresar un vector de condición inicial y una lista o tupla")
                    print("para los argumentos, recuerde el orden de estos.")
                    return None 
                elif ans2.replace(" ","").upper()=="SOL=SOLVE_IVP(F2,(0,T),X0,METHOD='RK45',ARGS=(ARGS,))":
                    print("Incorrecto, debe ocupar un método distinto a RK45.")
                    return None
                elif ans2.replace(" ","").upper() not in pos_answ2:
                    print("Incorrecto, recuerde que debe colocar los argumentos en el orden correcto")
                    print("y específicar que método va a utilizar.")
                    return None
                else:
                    print("Correcto!")
                    return None

    def solution(self):
        print("Se deja una solución que utiliza el método LSODA:")
        print()
        print("X0, args = [a0, m0, 1-a0-m0], (k1, k2, alpha)")
        print("sol = solve_ivp(F2, (0,T), X0, method='LSODA', args=(args,))")


import numpy as np 

class ex1:
    def __init__(self):
        self.intentos = 0
        pass

    def _Heun(self, f, ti, tf, N, X0, args):
        t = np.linspace(ti, tf, N)
        h = t[1] - t[0]
        ndim = len(X0)
        X = np.zeros((N,ndim))
        X[0] = X0
        for i in range(N-1):
            X_tilda = X[i] + h*f(t[i],X[i],args)
            X[i+1] = X[i] + h/2*(f(t[i],X[i],args)+f(t[i+1],X_tilda,args))
        return (t,X)

    def hint(self):
        print("Recuerde que para el método de Heun debe calcular primero un paso previo para y,")
        print("que llamamos y_tilda y luego actualizar la solución.")
        pass

    def check(self, y, f, ti, tf, N, y0, args):
        if np.var(y) != 0:
            self.intentos += 1
        correct = True
        t_ref, y_ref = self._Heun(f, ti, tf, N, y0, args)
        if len(y_ref) != len(y):
            print("Incorrecto, el largo de los arreglos no coincide.")
            print()
            correct = False
        else:
            for i in range(len(y_ref)):
                if abs(y[i][0]-y_ref[i][0])>0.01*abs(y_ref[i][0]) or abs(y[i][1]-y_ref[i][1])>0.01*abs(y_ref[i][1]):
                    print("Incorrecto, los valores no coinciden.")
                    print()
                    correct = False
                    break
        if correct:
            print("Correcto.")
            print()

        return (t_ref, y_ref)

    def solution(self):
        if self.intentos==0:
            print("Debe intertarlo al menos una vez.")
        else:
            print("Se deja un código que devuelve la solución esperada:")
            print()
            print("def Heun(f, ti, tf, N, X0, args):")
            print("    t = np.linspace(ti, tf, N)")
            print("    h = t[1] - t[0]")
            print("    ndim = len(X0)")
            print("    X = np.zeros((N,ndim))")
            print("    X[0] = X0")
            print("    for i in range(N-1):")
            print("        X_tilda = X[i] + h*f(t[i],X[i],args)")
            print("        X[i+1] = X[i] + h/2*(f(t[i],X[i],args)+f(t[i+1],X_tilda,args))")
            print("    return (t,X)")
        pass


from setuptools import setup 

setup(
    name="TN2601",
    version="0.5",
    description="Librería para poder chequear ejercicios propuestos del libro numérico.",
    author="Axel Osses A. & Diego Olguín W.",
    packages=["TN"],
    zip_safe=False
)


import numpy as np

class ex1:
    def __init__(self):
        self.intentos = 0
        pass

    def _Euler_Retrogrado(self, ti, tf, h, y0, args):
        R, alpha, gamma1, C = args
        N = int((tf-ti)/h)
        t = np.linspace(ti,tf,N)
        y_e = np.zeros(N)
        y_e[0] = y0
        for i in range(N-1):
            y_e[i+1] = y_e[i]/(1-h*(R-alpha*gamma1*C))
        return (t, y_e)

    def hint(self):
        print("Recuerde que debe generar una recurrencia particular para este problema.")
        print("Para ello reemplace f(t,T_e)=(R-alpha*gamma1*C)*T_e en la recurrencia para Euler retrógrado,")
        print("con esto podrá despejar T_e[i+1].")
        pass

    def check(self, y, ti, tf, h, y0, args):
        N = int((tf-ti)/h)
        if not np.var(y)==0:
            self.intentos += 1
        correct = True
        t_ref, y_ref = self._Euler_Retrogrado(ti, tf, h, y0, args)
        if len(y_ref) != len(y):
            print("Incorrecto, el largo de los arreglos no coincide.")
            print()
            correct = False
        else:
            for i in range(len(y_ref)):
                if abs(y[i]-y_ref[i])>0.1*abs(y_ref[i]):
                    print("Incorrecto, los valores no coinciden.")
                    print()
                    correct = False
                    break
        if correct:
            print("Correcto.")
            print()

        return (t_ref, y_ref)

    def solution(self):
        if self.intentos==0:
            print("Debe intentarlo al menos una vez")
        else:
            print("A continuación, se presenta un código que genera la solución:")
            print()
            print("def euler_retrogrado(ti, tf, h, y0, args):")
            print("    R, alpha, gamma1, C = args")
            print("    N = int((tf-ti)/h)")
            print("    t = np.linspace(ti,tf,N)")
            print("    y_e = np.zeros(N)")
            print("    y_e[0] = y0")
            print("    for i in range(N-1):")
            print("        y_e[i+1] = y_e[i]/(1-h*(R-alpha*gamma1*C))")
            print("    return (t, y_e)")
        pass